@@ -102,14 +102,8 @@ def _synchronize_node(configfile):
     os.remove(configfile)
     # Synchronize kitchen
     rsync_project(
-        node_work_path, './',
-        exclude=(
-            '/auth.cfg',  # might contain user credentials
-            '*.svn', '.bzr*', '.git*', '.hg*',  # ignore vcs data
-            '/cache/',
-            '/site-cookbooks/chef_solo_search_lib/'  # ignore data generated
-                                                     # by littlechef
-        ),
+        node_work_path, './cookbooks ./data_bags ./roles ./site-cookbooks',
+        exclude=('*.svn', '.bzr*', '.git*', '.hg*'),
         delete=True,
         extra_opts="-q",
     )
@@ -322,9 +316,14 @@ def _configure_node():
     cmd = 'chef-solo -l {0} -j /etc/chef/node.json'.format(env.loglevel)
     if ENABLE_LOGS:
         cmd += ' | tee {0}'.format(LOGFILE)
+    if env.loglevel == "debug":
+        print(
+            "Executing Chef Solo with the following command:\n{0}".format(cmd))
     with settings(hide('warnings', 'running'), warn_only=True):
         output = sudo(cmd)
-    if output.failed or "FATAL: Stacktrace dumped" in output:
+    if (output.failed or "FATAL: Stacktrace dumped" in output or
+            ("Chef Run complete" not in output and
+            "Report handlers complete" not in output)):
         if 'chef-solo: command not found' in output:
             print(
                 colors.red(
