@@ -16,7 +16,7 @@
 import os
 import simplejson as json
 import subprocess
-import re
+import imp
 
 from fabric import colors
 from fabric.api import env, settings
@@ -28,24 +28,8 @@ from littlechef.settings import cookbook_paths
 knife_installed = True
 
 
-def get_nodes(environment=None):
-    """Gets all nodes found in the nodes/ directory"""
-    if not os.path.exists('nodes/'):
-        return []
-    nodes = []
-    for filename in sorted([f for f in os.listdir('nodes/')
-                                if not os.path.isdir(f) and ".json" in f
-                                    and not f.startswith('.')]):
-        fqdn = ".".join(filename.split('.')[:-1])  # Remove .json from name
-        node = get_node(fqdn)
-        if environment is None or node.get('chef_environment') == environment:
-            # Add node name so that we can tell to which node it is
-            node['name'] = fqdn
-            nodes.append(node)
-    return nodes
-
-
 def get_node(name):
+    """Returns a JSON node file as a dictionary"""
     node_path = os.path.join("nodes", name + ".json")
     if not os.path.exists(node_path):
         return {'run_list': []}
@@ -57,9 +41,67 @@ def get_node(name):
             msg = 'LittleChef found the following error in'
             msg += ' "{0}":\n                {1}'.format(node_path, str(e))
             abort(msg)
+    # Add node name so that we can tell to which node it is
+    node['name'] = name
     return node
 
 
+def get_nodes(environment=None):
+    """Gets all nodes found in the nodes/ directory"""
+    if not os.path.exists('nodes/'):
+        return []
+    nodes = []
+    for filename in sorted([f for f in os.listdir('nodes/')
+                                if not os.path.isdir(f) and f.endswith(".json")
+                                    and not f.startswith('.')]):
+        fqdn = ".".join(filename.split('.')[:-1])  # Remove .json from name
+        node = get_node(fqdn)
+        if environment is None or node.get('chef_environment') == environment:
+            nodes.append(node)
+    return nodes
+
+
+def get_nodes_with_role(role_name, environment=None):
+    """Get all nodes which include a given role,
+    prefix-searches are also supported
+
+    """
+    prefix_search = role_name.endswith("*")
+    if prefix_search:
+        role_name = role_name.rstrip("*")
+    for n in get_nodes(environment):
+        roles = []
+        for role in get_roles_in_node(n):
+            roles.append(role)
+            roles.extend(get_roles_in_role(role))
+        if prefix_search:
+            if any(role.startswith(role_name) for role in roles):
+                yield n
+        else:
+            if role_name in roles:
+                yield n
+
+
+def get_nodes_with_recipe(recipe_name, environment=None):
+    """Get all nodes which include a given recipe,
+    prefix-searches are also supported
+
+    """
+    prefix_search = recipe_name.endswith("*")
+    if prefix_search:
+        recipe_name = recipe_name.rstrip("*")
+    for n in get_nodes(environment):
+        recipes = get_recipes_in_node(n)
+        for role in get_roles_in_node(n):
+            recipes.extend(get_recipes_in_role(role))
+        if prefix_search:
+            if any(recipe.startswith(recipe_name) for recipe in recipes):
+                yield n
+        else:
+            if recipe_name in recipes:
+                yield n
+
+
 def print_node(node, detailed=False):
     """Pretty prints the given node"""
     nodename = node['name']
@@ -85,6 +127,15 @@ def print_node(node, detailed=False):
         print "    {0}: {1}".format(attribute, node[attribute])
 
 
+def print_nodes(nodes, detailed=False):
+    """Prints all the given nodes"""
+    found = 0
+    for node in nodes:
+        found += 1
+        print_node(node, detailed=detailed)
+    print("\nFound {0} node{1}".format(found, "s" if found > 1 else ""))
+
+
 def _generate_metadata(path, cookbook_path, name):
     """Checks whether metadata.rb has changed and regenerate metadata.json"""
     global knife_installed
@@ -96,6 +147,9 @@ def _generate_metadata(path, cookbook_path, name):
         (not os.path.exists(metadata_path_json) or
          os.stat(metadata_path_rb).st_mtime > \
          os.stat(metadata_path_json).st_mtime)):
+        error_msg = "Warning: metadata.json for {0}".format(name)
+        error_msg += " in {0} is older that metadata.rb".format(cookbook_path)
+        error_msg += ", and cookbook attributes could be out of date\n"
         try:
             proc = subprocess.Popen(
                 ['knife', 'cookbook', 'metadata', '-o', cookbook_path, name],
@@ -103,22 +157,25 @@ def _generate_metadata(path, cookbook_path, name):
             resp, error = proc.communicate()
             if ('ERROR:' in resp or 'FATAL:' in resp \
                 or 'Generating metadata for' not in resp):
-                msg = "Unkown error while generating metadata.json for "
-                msg += "{0}. Cookbook attributes may be out of date".format(
-                    path)
-                print(msg)
+                if("No user specified, pass via -u or specifiy 'node_name'"
+                    in error):
+                    error_msg += "You need to have an up-to-date (>=0.10.x)"
+                    error_msg += " version of knife installed locally in order"
+                    error_msg += " to generate metadata.json.\nError "
+                else:
+                    error_msg += "Unkown error "
+                error_msg += "while executing knife to generate "
+                error_msg += "metadata.json for {0}".format(path)
+                print(error_msg)
                 if env.loglevel == 'debug':
                     print "\n".join(resp.split("\n")[:2])
             else:
                 print("Generated metadata.json for {0}".format(path))
         except OSError:
             knife_installed = False
-            msg = "Warning: metadata.json for {0}".format(name)
-            msg += " in {0} is older that metadata.rb".format(cookbook_path)
-            msg += ", and cookbook attributes could be out of date\n"
-            msg += "If you locally install Chef's knife tool, LittleChef will"
-            msg += " regenerate metadata files automatically"
-            print(msg)
+            error_msg += "If you locally install Chef's knife tool, LittleChef"
+            error_msg += " will regenerate metadata.json files automatically"
+            print(error_msg)
 
 
 def get_recipes_in_cookbook(name):
@@ -265,23 +322,6 @@ def get_roles():
     return sorted(roles, key=lambda x: x['fullname'])
 
 
-def get_nodes_with_role(rolename):
-    """Get all nodes which include a given role,
-    prefix-searches are also supported
-    """
-    prefix_search = rolename.endswith("*")
-    if prefix_search:
-        rolename = rolename.rstrip("*")
-    for n in get_nodes():
-        if prefix_search:
-            roles = get_roles_in_node(n)
-            if any(role.startswith(rolename) for role in roles):
-                yield n
-        else:
-            if rolename in get_roles_in_node(n):
-                yield n
-
-
 def print_role(role, detailed=True):
     """Pretty prints the given role"""
     if detailed:
@@ -299,6 +339,48 @@ def print_role(role, detailed=True):
     print("")
 
 
+def print_plugin_list():
+    """Prints a list of available plugins"""
+    print("List of available plugins:")
+    for plugin in get_plugins():
+        _pprint(plugin)
+
+
+def get_plugins():
+    """Gets available plugins by looking into the plugins/ directory"""
+    if os.path.exists('plugins/'):
+        for filename in sorted([f for f in os.listdir('plugins/')
+                                if not os.path.isdir(f) and f.endswith(".py")]):
+            plugin_name = filename[:-3]
+            try:
+                plugin = import_plugin(plugin_name)
+            except SystemExit as e:
+                description = "Plugin has a syntax error"
+            else:
+                description = plugin.__doc__ or "No description found"
+            yield {plugin_name: description}
+
+
+def import_plugin(name):
+    """Imports plugin python module"""
+    path = os.path.join("plugins", name + ".py")
+    try:
+        with open(path, 'rb') as f:
+            try:
+                plugin = imp.load_module(
+                    "p_" + name, f, name + '.py',
+                    ('.py', 'rb', imp.PY_SOURCE)
+                )
+            except SyntaxError as e:
+                error = "Found plugin '{0}', but it seems".format(name)
+                error += " to have a syntax error: {0}".format(str(e))
+                abort(error)
+    except IOError:
+        abort("Sorry, could not find '{0}.py' in the plugin directory".format(
+              name))
+    return plugin
+
+
 def get_cookbook_path(cookbook_name):
     """Returns path to the cookbook for the given cookbook name"""
     for cookbook_path in cookbook_paths:
@@ -336,26 +418,21 @@ def get_margin(length):
     return margin_left
 
 
-def parse_ip(text):
-    """Extract an IPv4 IP from a text string
-    Uses an IP Address Regex: http://www.regular-expressions.info/examples.html
-
-    """
-    ip_matches = re.findall(r'\b\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\b', text)
-    ip = ip_matches[0] if ip_matches else None
-    return ip
-
-
 def credentials(*args, **kwargs):
     """Override default credentials with contents of .ssh/config,
-    if appropriate
+
+    Only done if credentials are properly defined
     """
     if env.ssh_config:
         credentials = env.ssh_config.lookup(env.host)
-        # translate from paramiko params to fabric params
+        # translate from ssh params to fabric params
         if 'identityfile' in credentials:
             credentials['key_filename'] = credentials['identityfile']
         credentials.update(kwargs)
     else:
         credentials = kwargs
+    # Expand the home directory of 'key_filename', as 'ssh' won't do it
+    if credentials.get('key_filename'):
+        credentials['key_filename'] = os.path.expanduser(
+                                        credentials['key_filename'])
     return settings(*args, **credentials)
