@@ -21,7 +21,7 @@ import simplejson as json
 from fabric.api import *
 from fabric.contrib.files import append, exists
 from fabric.contrib.console import confirm
-from paramiko.config import SSHConfig as _SSHConfig
+from ssh.config import SSHConfig as _SSHConfig
 
 from littlechef import solo
 from littlechef import lib
@@ -115,11 +115,10 @@ def node(*nodes):
 
     # Check whether another command was given in addition to "node:"
     execute = True
-    if littlechef.__cooking__ and \
-        'node:' not in sys.argv[-1] and 'nodes_with_role:' not in sys.argv[-1]:
-        execute = False
-    # If user didn't type recipe:X, role:Y or deploy_chef, just run configure
-    if execute:
+    if not(littlechef.__cooking__ and
+            'node:' not in sys.argv[-1] and
+            'nodes_with_role:' not in sys.argv[-1]):
+        # If user didn't type recipe:X, role:Y or deploy_chef, just run configure
         for hostname in env.hosts:
             env.host = hostname
             env.host_string = hostname
@@ -134,7 +133,7 @@ def deploy_chef(gems="no", ask="yes", version="0.10",
     distro_type=None, distro=None, stop_client='yes'):
     """Install chef-solo on a node"""
     if not env.host_string:
-        abort('no node specified\nUsage: fix node:MYNODE deploy_chef')
+        abort('no node specified\nUsage: fix node:MYNODES deploy_chef')
     chef_versions = ["0.9", "0.10"]
     if version not in chef_versions:
         abort('Wrong Chef version specified. Valid versions are {0}'.format(
@@ -168,10 +167,11 @@ def recipe(recipe):
     """Apply the given recipe to a node
     Sets the run_list to the given recipe
     If no nodes/hostname.json file exists, it creates one
+
     """
     # Check that a node has been selected
     if not env.host_string:
-        abort('no node specified\nUsage: fix node:MYNODE recipe:MYRECIPE')
+        abort('no node specified\nUsage: fix node:MYNODES recipe:MYRECIPE')
     lib.print_header(
         "Applying recipe '{0}' on node {1}".format(recipe, env.host_string))
 
@@ -186,10 +186,11 @@ def role(role):
     """Apply the given role to a node
     Sets the run_list to the given role
     If no nodes/hostname.json file exists, it creates one
+
     """
     # Check that a node has been selected
     if not env.host_string:
-        abort('no node specified\nUsage: fix node:MYNODE role:MYROLE')
+        abort('no node specified\nUsage: fix node:MYNODES role:MYROLE')
     lib.print_header(
         "Applying role '{0}' to {1}".format(role, env.host_string))
 
@@ -200,67 +201,58 @@ def role(role):
         chef.sync_node(data)
 
 
-@hosts('setup')
-def get_ips():
-    """Ping all nodes and update their 'ipaddress' field"""
-    import subprocess
-    for node in lib.get_nodes():
-        # For each node, ping the hostname
-        env.host_string = node['name']
-        proc = subprocess.Popen(['ping', '-c', '1', node['name']],
-                    stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        resp, error = proc.communicate()
-        if not error:
-            # Split output into lines and parse the first line to get the IP
-            ip = lib.parse_ip(resp.split("\n")[0])
-            if not ip:
-                print "Warning: could not get IP address from node {0}".format(
-                    node['name'])
-                continue
-            print "Node {0} has IP {1}".format(node['name'], ip)
-            # Update with the ipaddress field in the corresponding node.json
-            del node['name']
-            node['ipaddress'] = ip
-            os.remove(chef.save_config(node, ip))
+def ssh(name):
+    """Executes the given command"""
+    if not env.host_string:
+        abort('no node specified\nUsage: fix node:MYNODES ssh:COMMAND')
+    print("\nExecuting the command '{0}' on the node {1}...".format(
+          name, env.host_string))
+    # Execute remotely using either the sudo or the run fabric functions
+    with settings(hide("warnings"), warn_only=True):
+        if name.startswith("sudo "):
+            sudo(name[5:])
         else:
-            print "Warning: could not resolve node {0}".format(node['name'])
+            run(name)
+
+
+def plugin(name):
+    """Executes the selected plugin
+    Plugins are expected to be found in the kitchen's 'plugins' directory
+
+    """
+    if not env.host_string:
+        abort('No node specified\nUsage: fix node:MYNODES plugin:MYPLUGIN')
+    plug = lib.import_plugin(name)
+    print("Executing plugin '{0}' on {1}".format(name, env.host_string))
+    node = lib.get_node(env.host_string)
+    if node == {'run_list': []}:
+        node['name'] = env.host_string
+    plug.execute(node)
+    print("Finished executing plugin")
 
 
 @hosts('api')
 def list_nodes():
     """List all configured nodes"""
-    for node in lib.get_nodes(env.chef_environment):
-        lib.print_node(node)
+    lib.print_nodes(lib.get_nodes(env.chef_environment))
 
 
 @hosts('api')
 def list_nodes_detailed():
     """Show a detailed list of all nodes"""
-    for node in lib.get_nodes(env.chef_environment):
-        lib.print_node(node, detailed=True)
+    lib.print_nodes(lib.get_nodes(env.chef_environment), detailed=True)
 
 
 @hosts('api')
 def list_nodes_with_recipe(recipe):
     """Show all nodes which have asigned a given recipe"""
-    for node in lib.get_nodes(env.chef_environment):
-        if recipe in lib.get_recipes_in_node(node):
-            lib.print_node(node)
-        else:
-            for role in lib.get_roles_in_node(node):
-                with open('roles/' + role + '.json', 'r') as f:
-                    roles = json.loads(f.read())
-                    # Reuse _get_recipes_in_node to extract recipes in a role
-                    if recipe in lib.get_recipes_in_node(roles):
-                        lib.print_node(node)
-                        break
+    lib.print_nodes(lib.get_nodes_with_recipe(recipe, env.chef_environment))
 
 
 @hosts('api')
 def list_nodes_with_role(role):
     """Show all nodes which have asigned a given role"""
-    for node in lib.get_nodes_with_role(role):
-        lib.print_node(node)
+    lib.print_nodes(lib.get_nodes_with_role(role, env.chef_environment))
 
 
 @hosts('api')
@@ -296,6 +288,12 @@ def list_roles_detailed():
         lib.print_role(role)
 
 
+@hosts('api')
+def list_plugins():
+    """Show all available plugins"""
+    lib.print_plugin_list()
+
+
 # Check that user is cooking inside a kitchen and configure authentication #
 def _check_appliances():
     """Look around and return True or False based on whether we are in a
@@ -378,6 +376,8 @@ def _readconfig():
 import littlechef
 env.chef_environment = littlechef.chef_environment
 env.loglevel = littlechef.loglevel
+env.verbose = littlechef.verbose
+
 
 if littlechef.__cooking__:
     # Called from command line
