@@ -84,7 +84,8 @@ def configure(current_node=None):
         data = {
             'node_work_path': node_work_path,
             'cookbook_paths_list': cookbook_paths_list,
-            'environment': current_node.get('chef_environment', '_default')
+            'environment': current_node.get('chef_environment', '_default'),
+            'verbose': "true" if env.verbose else "false"
         }
         with hide('running', 'stdout'):
             upload_template(os.path.join(BASEDIR, 'solo.rb'), '/etc/chef/',
